// include only avr libraries
#include <avr/io.h>
#include <util/delay.h>

#define BUS_MASK 0x0F
#define dbg

uint8_t mtc[] = {'H','e','l','l','o',' ','w','o','r','l','d','!'}; // to check

uint8_t msg[13]; // message received (note: 13 for null terminator)
const uint8_t msg_len = 24; // in symbols
uint8_t ri = 0;  // read index
uint8_t reading = 1;  // start in reading state

void check_function();
void receive_function();

void setup() {
#ifdef dbg
  Serial.begin(115200);
  while(!Serial);
  msg[12] = '\0';
#endif
  DDRD &= ~_BV(DDB4); // DATA_READY at pin PD4
  DDRB &= ~BUS_MASK;  // our bus at PB0 to PB3
  DDRB |= _BV(DDB5);  // use the LED on PB5
}

void loop() {
  receive_function();
  check_function();
}



// the function to receive symbols...
void receive_function() {
  if(ri<msg_len){
    if(PIND & _BV(PIND4)){ //read data ready signal
      uint8_t str = PINB & BUS_MASK;  //read the symbol
#ifdef dbg
      Serial.println(str,HEX);
#endif
      uint8_t mi = ri >> 1;
      msg[mi] |= (ri % 2 == 0) ? str : str << 4;
    
      // 3 short blinks on led
      PORTB |= _BV(PORTB5); _delay_ms(25);  PORTB &= ~_BV(PORTB5); _delay_ms(25);
      PORTB |= _BV(PORTB5); _delay_ms(25);  PORTB &= ~_BV(PORTB5); _delay_ms(25);
      PORTB |= _BV(PORTB5); _delay_ms(25);  PORTB &= ~_BV(PORTB5); _delay_ms(25);

      ri++;                     //increment read index
    }
  }
}

void check_function() {
  if(ri==msg_len){
#ifdef dbg
    Serial.println((char *)msg);
#endif
    _delay_ms(500);
    uint8_t ok = 1;
    for(uint8_t i; i<(msg_len>>1); i++){
      ok &= (msg[i]==mtc[i]);
    }
    if(ok){
      while(1){ // slow blinks forever
        PORTB |= _BV(PORTB5); _delay_ms(1000);  PORTB &= ~_BV(PORTB5); _delay_ms(1000);
      }
    } else {
      while(1){ // ERROR: fast blinks forever
        PORTB |= _BV(PORTB5); _delay_ms(25);  PORTB &= ~_BV(PORTB5); _delay_ms(25);
      }
    }
  }
}

