// include only avr libraries
#include <avr/io.h>
#include <util/delay.h>

#define BUS_MASK 0x0F
#define dbg

uint8_t mtc[] = {'H','e','l','l','o',' ','w','o','r','l','d','!'}; // to check
uint8_t mts[] = {'H','e','l','l','o',' ','w','o','r','l','d','!'}; //12 x 2 sym

uint8_t msg[13]; // message received (note: 13 for null terminator)
const uint8_t msg_len = 24; // in symbols
uint8_t wi = 0;  // write index
uint8_t ri = 0;  // read index
uint8_t reading = 1;  // start in reading state

void check_task();
void receive_task();
void send_task();

void setup() {
#ifdef dbg
  Serial.begin(115200);
  while(!Serial);
  msg[12] = '\0';
#endif
  pinMode(13, OUTPUT);
  //sender
  pinMode(4, OUTPUT);
  pinMode(8, OUTPUT); pinMode(9, OUTPUT);pinMode(10, OUTPUT);pinMode(11, OUTPUT);
  digitalWrite(4, LOW);
  PORTB &= ~BUS_MASK; //clear the bus
  //receiver
  pinMode(18, INPUT);
  pinMode(14, INPUT); pinMode(15, INPUT);pinMode(16, INPUT);pinMode(17, INPUT);
}

void loop() {
  send_task();
  receive_task();
  delay(400); // to see the different transfers on blink
  check_task();
}

// the function to send symbols...
void send_task() {
  if(wi<msg_len){
    uint8_t mi = wi >> 1;
    uint8_t stw = (wi % 2 == 0) ? mts[mi] : mts[mi] >> 4; //local var symbol
    PORTB &= ~BUS_MASK; //clear the bus
    PORTB |= stw & BUS_MASK; //write the symbol
    delay(5);
    digitalWrite(4, HIGH);
  
    //short blink on led
    digitalWrite(13, HIGH); delay(50); digitalWrite(13, LOW); delay(50);
  
    wi++;                     //increment write index
    digitalWrite(4, LOW);
  }
}

// the function to receive symbols... now is an ISR
void receive_task() {
  if(ri<msg_len){
    if(digitalRead(18)){ //read data ready signal
      uint8_t str = PINC & BUS_MASK;  //read the symbol
#ifdef dbg
      Serial.println(str,HEX);
#endif
      uint8_t mi = ri >> 1;
      msg[mi] |= (ri % 2 == 0) ? str : str << 4;
    
      // 3 short blinks on led
    digitalWrite(13, HIGH); delay(25); digitalWrite(13, LOW); delay(25);
    digitalWrite(13, HIGH); delay(25); digitalWrite(13, LOW); delay(25);
    digitalWrite(13, HIGH); delay(25); digitalWrite(13, LOW); delay(25);

      ri++;                     //increment read index
    }
  }
}

void check_task() {
  if(ri==msg_len){
#ifdef dbg
    Serial.println((char *)msg);
#endif
    delay(500);
    uint8_t ok = 1;
    for(uint8_t i; i<(msg_len>>1); i++){
      ok &= (msg[i]==mtc[i]);
    }
    if(ok){
      while(1){ // slow blinks forever
        digitalWrite(13, HIGH); delay(1000); digitalWrite(13, LOW); delay(1000);
      }
    } else {
      while(1){ // ERROR: fast blinks forever
        digitalWrite(13, HIGH); delay(25); digitalWrite(13, LOW); delay(25);
      }
    }
  }
}

