// include only avr libraries
#include <avr/io.h>
#include <util/delay.h>

#define BUS_MASK 0x0F
#define NO_API

uint8_t mts[] = {'E','m','b','e','d','d','e','d',' ','S','y','s','t','e','m','s','\n','C','4','u','C'}; //21 x 2 sym
uint8_t minit[] = {0x33, 0x32, 0x28, 0x0C, 0x06, 0x01}; //6 x 2 sym

//             DB: 7654 3210
//                 |||| ||||
// $33 8-bit mode  xx11 0011  <-- Init sequence to switch to 4bit mode...
// $32 8-bit mode  xx11 0010  <-- ... three times 0x11 and one 0x10
// $28 8-bit mode  xx10 1000  <-- Function set: 4bit bus, 2-line, 5x8 dots
// $0C 8-bit mode  xx00 1100  <-- Display ctrl: display ON, cursor OFF, blink OFF
// $06 8-bit mode  xx00 0110  <-- Entry mode: cursor moves to right
// $01 8-bit mode  xx00 0001  <-- Clear display

const uint8_t msg_len = 42; // in symbols
const uint8_t init_len = 12; // in symbols
uint8_t wi = 0;  // write index
uint8_t ii = 0;  // init index

void send_task();

#ifndef NO_API
void setup() {
#else
int main() {
#endif
  DDRD |= _BV(DDD4); // Enable
  DDRD |= _BV(DDD5); // Register Select
  DDRB |= BUS_MASK;
  PORTB &= ~BUS_MASK; //clear the bus

#ifdef NO_API
  while(1){
    send_task();
  } 
#endif
}

#ifndef NO_API
void loop() {
  send_task();
}
#endif

// the function to send symbols...
void send_task() {
  if(ii<init_len){
    uint8_t mi = ii >> 1;
    uint8_t stw = (ii % 2 == 0) ? minit[mi] >> 4 : minit[mi]; //local var symbol

    PORTD &= ~_BV(PORTD5);   // cmd_mode rs=0
    PORTB &= ~BUS_MASK;      //clear the bus
    PORTB |= stw & BUS_MASK; // write the symbol

    //short blink on led
    PORTB |= _BV(PORTB5); _delay_ms(50); PORTB &= ~_BV(PORTB5); _delay_ms(50);
  
    PORTD |= _BV(PORTD4);    // signal data on bus, enable=1
    _delay_ms(1);
    PORTD &= ~_BV(PORTD4);   // enable=0
  
    ii++;                    // increment init index
    return;
  }
  
  if(wi<msg_len){
    uint8_t mi = wi >> 1;

    if(mts[mi]=='\n'){ // small workaround to write next line cmd during chars transmission 
      wi += 2;
      ii -= 2;
      minit[ii>>1] = 0xC0; // next line
      return;
    }
    
    uint8_t stw = (wi % 2 == 0) ? mts[mi] >> 4 : mts[mi]; //local var symbol
    
    PORTD |= _BV(PORTD5);    // char_mode rs=1
    PORTB &= ~BUS_MASK;      // clear the bus
    PORTB |= stw & BUS_MASK; // write the symbol

    //short blink on led
    PORTB |= _BV(PORTB5); _delay_ms(50); PORTB &= ~_BV(PORTB5); _delay_ms(50);
  
    PORTD |= _BV(PORTD4);    // signal data on bus, enable=1
    _delay_ms(1);
    PORTD &= ~_BV(PORTD4);   // enable=0
  
    wi++;                    // increment write index
  }
} //end of send_task




