#include <avr/io.h>
#include <util/delay.h>

#define FOSC 16000000L // Clock Speed
#define BAUD 9600
#define MYUBRR FOSC/16/BAUD-1  //See table 24-1 datasheet p.227

void USART_Init( unsigned int ubrr);
void USART_Transmit( unsigned char data );
unsigned char USART_Receive( void );
bool USART_Overrun( void );
bool USART_ClearOverrun( void );

uint8_t test = 3;

int main( void ){
  
  DDRB |= _BV(DDB5); // Led
  USART_Init(MYUBRR);
  unsigned char msg[] = "Hello UART!";
  unsigned char msg2[] = "You wrote ";
  uint8_t msg_len = 11;
  uint8_t msg2_len = 10;

  unsigned char input = 0;

  bool overrunOccurred = 0;

  switch(test){
    case 1:
      for(uint8_t i=0; i<msg_len;i++){
        USART_Transmit(msg[i]);
      }
      break;
    case 2:
      while(1){
        input = USART_Receive();
        USART_Transmit(input);
      }
      break;
    default:
      while(1){
        input = USART_Receive();
        
        if(USART_Overrun()) overrunOccurred = 1;
        if(input == 'c') overrunOccurred = USART_ClearOverrun();
        
        if (overrunOccurred){ PORTB |= _BV(PORTB5);}
        else { PORTB &= ~_BV(PORTB5);}
        
        for(uint8_t i = 0; i< msg2_len; i++){
          USART_Transmit(msg2[i]);
        }
        USART_Transmit(input);
        USART_Transmit(0x0A); //0x0A = 10 = '\n'
      }
  }  
}

//NOTE "=" and not "|=" or "&="
void USART_Init( unsigned int ubrr){
  /*Set baud rate */
  UBRR0H = (unsigned char)(ubrr>>8); 
  UBRR0L = (unsigned char)ubrr;
  /*Enable receiver and transmitter */
  UCSR0B = (1<<RXEN0)|(1<<TXEN0);
  /* Set frame format: 8data, 1stop bit */
  UCSR0C = (3<<UCSZ00);
}

void USART_Transmit( unsigned char data ){
  /* Wait for empty transmit buffer */
  while ( !( UCSR0A & (1<<UDRE0)) ) ;
  /* Put data into buffer, sends the data */
  UDR0 = data;
}

unsigned char USART_Receive( void ){
  /* Wait for data to be received */
  while ( !(UCSR0A & (1<<RXC0)) ) ;
  /* Get and return received data from buffer */
  return UDR0;
}

bool USART_Overrun( void ){
  return (bool) (UCSR0A & (1<<DOR0));
}

bool USART_ClearOverrun( void ){
  unsigned char dummy;
  UCSR0A &= ~(1<<DOR0); //clear overrun flag, needed?!?!
  while ( UCSR0A & (1<<RXC0) ) dummy = UDR0; //flushing the buffer
  return 0;
}

