#ifndef SIMPLE_UART_H
#define SIMPLE_UART_H
#include <avr/io.h>

#define F_CPU 16000000UL
#define BAUD 115200
#include <util/setbaud.h>
#include <util/delay.h>

void uart_write(char c);
char uart_read();
uint8_t m_uart_data_available();
void uart_echo();
void uart_init(void);

void uart_init(void){
  
  // FOSC/16/BAUD-1
  UBRR0H = UBRRH_VALUE;
  UBRR0L = UBRRL_VALUE;
  #if USE_2X
  UCSR0A |= _BV(U2X0);
  #else
  UCSR0A &= ~(_BV(U2X0));
  #endif
  UCSR0C = _BV(UCSZ01) | _BV(UCSZ00);
  UCSR0B = _BV(RXEN0) | _BV(TXEN0);
}

void uart_write(char c)
{
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
}

char uart_read()
{
  loop_until_bit_is_set(UCSR0A, RXC0);
  return UDR0;
}

uint8_t m_uart_data_available()
{
  //loop_until_bit_is_set(UCSR0A, RXC0);
  return (UCSR0A & _BV(RXC0)) >> RXC0;
}

void uart_echo(){
  loop_until_bit_is_set(UCSR0A, RXC0);
  char x = UDR0;
  _delay_ms(10);
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = x;
}

#endif /* SIMPLE_UART_H */

