#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>

void USART_Init( unsigned int ubrr);
void USART_Transmit( unsigned char data );
unsigned char USART_Receive( void );
bool USART_Overrun( void );
bool USART_ClearOverrun( void );

void USART_print(unsigned char * msg, unsigned int len);
void USART_println(unsigned char * msg, unsigned int len);
void USART_println_uHex(unsigned int uint);

//NOTE "=" and not "|=" or "&="
void USART_Init( unsigned int ubrr){
  /*Set baud rate */
  UBRR0H = (unsigned char)(ubrr>>8); 
  UBRR0L = (unsigned char)ubrr;
  /*Enable receiver and transmitter */
  UCSR0B = (1<<RXEN0)|(1<<TXEN0);
  /* Set frame format: 8data, 1stop bit */
  UCSR0C = (3<<UCSZ00);
}

void USART_Transmit( unsigned char data ){
  /* Wait for empty transmit buffer */
  while ( !( UCSR0A & (1<<UDRE0)) ) ;
  /* Put data into buffer, sends the data */
  UDR0 = data;
}

unsigned char USART_Receive( void ){
  /* Wait for data to be received */
  while ( !(UCSR0A & (1<<RXC0)) ) ;
  /* Get and return received data from buffer */
  return UDR0;
}

bool USART_Overrun( void ){
  return (bool) (UCSR0A & (1<<DOR0));
}

bool USART_ClearOverrun( void ){
  unsigned char dummy;
  UCSR0A &= ~(1<<DOR0); //clear overrun flag, needed?!?!
  while ( UCSR0A & (1<<RXC0) ) dummy = UDR0; //flushing the buffer
  return 0;
}

void USART_print(unsigned char * msg, unsigned int len){
  for(uint8_t i=0; i<len;i++){
    if(msg[i]==0)return;
    USART_Transmit(msg[i]);
  }
}
void USART_println(unsigned char * msg, unsigned int len){
  USART_print(msg, len);
  USART_Transmit('\n');
  
}
void USART_println_uHex(unsigned int uint){
  char buf[2+sizeof(unsigned int)*8+1] = { 0 };
  buf[0] = '0'; buf[1] = 'x'; 
  utoa(uint, buf+2, 16);
  USART_println((unsigned char *)buf, sizeof(buf));
}



