// I2C Scanner,  Written by us @ Embedded Systems course

#include <util/twi.h>
#include <util/delay.h>
#include "avr_uart.h"     //our AVR uart example inside a .h file (+some util print functs)

#define FOSC 16000000L // Clock Speed
#define BAUD 9600
#define MYUBRR FOSC/16/BAUD-1  //See table 24-1 datasheet p.227

#define MYTWI_FREQ 100000UL

int main( void ){
  
  DDRB |= _BV(DDB5); // Led
  USART_Init(MYUBRR);
  unsigned char msg[] = "Device at ";
  uint8_t msg_len = 10;

  // activate internal pullups for twi.
  PORTC = _BV(DDC4) | _BV(DDC5);

  // initialize twi prescaler and bit rate
  TWSR &= ~_BV(TWPS0);
  // SCL Frequency = CPU Clock Frequency / (16 + (2 * TWBR))
  // note: TWBR should be 10 or higher for master mode
  // It is 72 for a 16mhz Wiring board with 100kHz TWI
  TWBR = ((FOSC / MYTWI_FREQ) - 16) / 2;
  
  // enable twi module, acks
  TWCR = _BV(TWEN) | _BV(TWEA);  
  uint8_t sla;
  for (uint8_t i = 8; i < 127; i++){
    TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);  // Send START condition
    while (!(TWCR &(1<<TWINT)));             // Wait for TWINT Flag set. This indicates that the START condition has been transmitted.
    if ((TWSR & 0xF8) != TW_START)           // Check value of TWI Status Register. Mask prescaler bits. 
      PORTB |= _BV(PORTB5);                  // If status different from START turn on the LED
    sla = ((i & 0x7F) << 1) | TW_WRITE;      // Shift sleave address one left (for the frame) and set TW_WRITE flag
    TWDR = sla;                              // Load slave address into TWDR Register.
    TWCR = (1<<TWINT) | (1<<TWEN);           // Clear TWINT bit in TWCR to start transmission of address.
    while (!(TWCR & (1<<TWINT)));            // Wait for TWINT Flag set. This indicates that the SLA+W has been transmitted, 
                                             // and ACK/NACK has been received.
    if ((TWSR & 0xF8) == TW_MT_SLA_ACK){     // ACK, a device is on the bus...
      USART_print(msg, msg_len);             // ... print the message...
      USART_println_uHex(i);                 // ... and the device address
    }
    TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);  // Transmit STOP condition
    
    _delay_ms(1);
  }
}
