#include "Wire.h"
#define DS3231_I2C_ADDRESS 0x68

unsigned long st_time;
unsigned long int_time_1;
unsigned long int_time_2;
uint8_t pin_state;
volatile uint8_t int_occurred_up = 0;
volatile uint8_t int_occurred_down = 0;

// Convert normal decimal numbers to binary coded decimal
byte decToBcd(byte val)
{
  return( (val/10*16) + (val%10) );
}
// Convert binary coded decimal to normal decimal numbers
byte bcdToDec(byte val)
{
  return( (val/16*10) + (val%16) );
}

void serve_int(){
  if(digitalRead(2)==0){
    int_time_1 = millis();
    int_occurred_down = 1;
  } else {
    int_time_2 = millis();
    int_occurred_up = 1;
  }
}

void setup()
{
  pinMode(13, OUTPUT);
  pinMode(2, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(2), serve_int, CHANGE);
  Wire.begin();
  Serial.begin(115200);
  while(!Serial);
  // set the initial time here:
  // DS3231 seconds, minutes, hours, day, date, month, year
//  Serial.println("Send a char to synch to 11:47:00");
//  while(!Serial.available());
//  int w = Serial.parseInt();
//   setDS3231time(0,47,11,3,23,10,18);

  
  readDS3231_regs();
  delay(100);
  setDS3231_ALM2_in_next_minutes(1);
  delay(100);
  setDS3231_ctrls();
  
  
  st_time = millis();
}

void readDS3231_regs(){
  uint8_t regs[16] = {};
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x00); // set next input to start at register 0
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 16);
  for(uint8_t i=0; i<16; i++){
    regs[i] = Wire.read();
  }
  for(uint8_t i=0; i<16; i++){
    Serial.print("Reg ["); Serial.print(i,HEX) ;Serial.print("]: "); Serial.println(regs[i], HEX);
  }
}

void setDS3231_ctrls(){
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x0E); // set next input to start at the control register
  Wire.write(0x06);
  Wire.endTransmission();
  
}

void setDS3231_ALM2_in_next_minutes(uint8_t mins){
  //get current mins...
  uint8_t minute;
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x01); // set DS3231 register pointer to 01h
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 1);
  minute = bcdToDec(Wire.read());
  Serial.print("Curr.min="); Serial.println(minute) ;
  
  //add mins...
  minute = (minute + mins) % 60;
  Serial.print("Next alm.min="); Serial.println(minute) ;
  
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0x0B); // set next input to start at 0x0B register
  Wire.write(decToBcd(minute)); // set minutes at 0x0B
  Wire.write(0x80); // set reg at 0x0C
  Wire.write(0x80); // set reg at 0x0D
  Wire.write(0x06); // set reg at 0x0E
  Wire.write(0x88); // set reg at 0x0F
  Wire.endTransmission();
  }


void setDS3231time(byte second, byte minute, byte hour, byte dayOfWeek, byte dayOfMonth, byte month, byte year){
  // sets time and date data to DS3231
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0); // set next input to start at the seconds register
  Wire.write(decToBcd(second)); // set seconds
  Wire.write(decToBcd(minute)); // set minutes
  Wire.write(decToBcd(hour)); // set hours
  Wire.write(decToBcd(dayOfWeek)); // set day of week (1=Sunday, 7=Saturday)
  Wire.write(decToBcd(dayOfMonth)); // set date (1 to 31)
  Wire.write(decToBcd(month)); // set month
  Wire.write(decToBcd(year)); // set year (0 to 99)
  Wire.endTransmission();
}
void readDS3231time(byte *second,
byte *minute,
byte *hour,
byte *dayOfWeek,
byte *dayOfMonth,
byte *month,
byte *year)
{
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0); // set DS3231 register pointer to 00h
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 7);
  // request seven bytes of data from DS3231 starting from register 00h
  *second = bcdToDec(Wire.read() & 0x7f);
  *minute = bcdToDec(Wire.read());
  *hour = bcdToDec(Wire.read() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.read());
  *dayOfMonth = bcdToDec(Wire.read());
  *month = bcdToDec(Wire.read());
  *year = bcdToDec(Wire.read());
}
void displayTime()
{
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  // retrieve data from DS3231
  readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,
  &year);
  // send it to the serial monitor
  Serial.print(hour, DEC);
  // convert the byte variable to a decimal number when displayed
  Serial.print(":");
  if (minute<10)
  {
    Serial.print("0");
  }
  Serial.print(minute, DEC);
  Serial.print(":");
  if (second<10)
  {
    Serial.print("0");
  }
  Serial.print(second, DEC);
  Serial.print(" ");
  Serial.print(dayOfMonth, DEC);
  Serial.print("/");
  Serial.print(month, DEC);
  Serial.print("/");
  Serial.print(year, DEC);
  Serial.print(" Day of week: ");
  switch(dayOfWeek){
  case 1:
    Serial.println("Sunday");
    break;
  case 2:
    Serial.println("Monday");
    break;
  case 3:
    Serial.println("Tuesday");
    break;
  case 4:
    Serial.println("Wednesday");
    break;
  case 5:
    Serial.println("Thursday");
    break;
  case 6:
    Serial.println("Friday");
    break;
  case 7:
    Serial.println("Saturday");
    break;
  }
}

unsigned long alm_reset_delayed;
uint8_t alarm_active = 0;

void loop()
{
  if(st_time + 5000 < millis()){
    st_time = millis();
    displayTime(); // display the real-time clock data on the Serial Monitor,
  }

  if(int_occurred_down){
    st_time = millis();
    Serial.print("INT down at: ");
    Serial.println(int_time_1);
    int_occurred_down = 0;
    alm_reset_delayed = millis();
    alarm_active = 1;
    //setDS3231_ALM2_in_next_minutes(1);
  }
  
  if((alarm_active)&(alm_reset_delayed + 20000 < millis())){
    alarm_active = 0;
    setDS3231_ALM2_in_next_minutes(1);
  }
  
  if(int_occurred_up){
    Serial.print("INT up at: ");
    Serial.println(int_time_2);
    int_occurred_up = 0;
  }
  
  pin_state = digitalRead(2);
  digitalWrite(13, pin_state);

  if(Serial.available()>0){
    uint8_t c = Serial.read();
    if(c=='r') readDS3231_regs();
  }
  delay(5); // 
}
