#include <avr/io.h>
#include <avr/delay.h>

#define ADC_PIN 0          // ADC 0
#define ADC_THRESHOLD 512  // Our threshold for turning the LED ON or OFF

int main(void){
  Serial.begin(115200);      // This is an "abuse" here... :-)
  while (!Serial);
  
  DDRB  |= _BV(PORTB5);
  ADCSRA |= _BV(ADEN);     // Enable the ADC, see datasheet section 28.9.2

  // Set reference voltage to Vcc (REFSn=01), see datasheet section 28.9.1, table 28-3
  ADMUX |= (1<<REFS0);

  // Set the lower 3 bits of the MUX register to the pin number, 
  // see DS section 28.9.1, table 28-4
  ADMUX &= 0xF0;
  ADMUX |= ADC_PIN;

  // Set freq prescaler to 128 
  // See datasheet section 28.9.1, table 28-5 
  ADCSRA |= 0b00000111; // Or, in other words.. (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0) 

  while (1) {
    uint16_t value = 0;

    ADCSRA |= _BV(ADSC);             // Start conversion
    
    while ( (ADCSRA & _BV(ADSC)) );  // wait until completion

    // we could recreate the 10-bit value...
    //     value = ((ADCH & 0b00000011) << 8) | ADCL0;
    // ...but we can use the ADC macro 
    value = ADC;
    
    Serial.print(value);
    Serial.print('\n');
    
    if (value > ADC_THRESHOLD)
      PORTB |= _BV(PORTB5);
    else
      PORTB &= ~_BV(PORTB5);

    _delay_ms(100);
  }
}
